package org.eclipse.jetty.rt.example.test;

import org.eclipse.jetty.client.*;
import org.eclipse.jetty.util.Attributes;
import org.eclipse.jetty.http.HttpBuffers;
import org.eclipse.jetty.util.component.AbstractLifeCycle;
import org.eclipse.jetty.http.HttpMethods;
import org.eclipse.jetty.http.HttpStatus;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.FrameworkUtil;

/**
 * 
 */
public class ServletTest {

	private HttpClient _client;
	
    protected void startClient() throws Exception {
	    _client = new HttpClient();
	    _client.setConnectorType(HttpClient.CONNECTOR_SELECT_CHANNEL);
	    _client.start();
    }
    
	protected void stopClient() throws Exception {
		if (_client != null) {
			_client.stop();
			_client = null;
		}
	}
	
	/**
	 * Make sure the webapp is started before this test is run.
	 * Could be done via the Launch Configuration
	 */
	@Before public void setup() {
		Bundle testBundle = FrameworkUtil.getBundle(ServletTest.class);
		for (Bundle b : testBundle.getBundleContext().getBundles()) {
			if (b.getSymbolicName().equals("org.eclipse.jetty.osgi.boot")
					|| b.getSymbolicName().equals("org.eclipse.jetty.example.webapp")) {
				try {
					System.err.println("Found " + b.getSymbolicName());
					b.start();
				} catch (BundleException e) {
					Assert.fail("Unable to start the necessary bundles");
				}
			}
		}
		//TODO: put in a lifecycle listener
		synchronized (Thread.currentThread()) {
			try {
				Thread.currentThread().wait(1000);
			} catch (Throwable e) {
				e.printStackTrace();
			}
		}
	}
    
	@Test public void testGet() throws Exception {
		startClient();

		ContentExchange getExchange = new ContentExchange();
		getExchange.setURL("http://localhost:8080/test/");
		getExchange.setMethod(HttpMethods.GET);

		_client.send(getExchange);
		int state = getExchange.waitForDone();

		String content = "";
		int responseStatus = getExchange.getResponseStatus();
		if (responseStatus == HttpStatus.OK_200) {
			content = getExchange.getResponseContent();
		}
		System.err.println("Got the content: " + content);
		stopClient();

		Assert.assertEquals(HttpStatus.OK_200, responseStatus);
		Assert.assertEquals("Howdy!", content);
	}

}
